@@ -1,10 +1,11 @@
-# /usr/bin/python3
+#!/usr/bin/python3
 # -*- coding: utf-8 -*-
 """ Модуль авторизации в Полиматике """
+
 import requests
-import time
 import json
 from typing import Tuple, Dict
+from .exceptions import AuthError
 
 
 class Authorization:
@@ -18,10 +19,8 @@ class Authorization:
         :param url: базовый URL стенда
         :param server_codes: значение файла "server_codes.json" (хранит коды команд и их состояний)
         :param language: "en" / "ru" / "de" / "fr"
-        :return: (Tuple) session_id, uuid/manager_id, func_timing
+        :return: (Tuple) идентификатор сессии, uuid, полная версия Полиматики
         """
-        start_time = time.time()
-
         # для авторизации с паролем
         session = ""
 
@@ -31,19 +30,21 @@ class Authorization:
         auth_login = auth_manager_command.get("state", {}).get("login")
         language = server_codes.get("locale", {}).get(language)
 
-        # для авторизации без пароля
+        # для авторизации без пароля формируем id сессии для беспарольного пользователя
         if password is None:
-            # формирование id сессии
-            url = url + "/login"
-            payload = {"login": user_name}
-            r = requests.get(url=url, params=payload)
-            try:
-                response = r.json()
-            except json.decoder.JSONDecodeError:
-                message = "Host {} not supporting non-password authorization. Please, specify the password!".format(
-                    url
-                )
-                raise ValueError({'message': message})
+            login_url = "{}login".format(url)
+            r = requests.get(url=login_url, params={"login": user_name})
+            if not r.ok:
+                msg = 'Invalid server response (URL: {}, Code: {}, Reason: "{}", Text: {})'.format(
+                    login_url, r.status_code, r.reason, '<empty>' if not r.text else r.text)
+                raise AuthError(msg)
+            else:
+                try:
+                    response = r.json()
+                except json.decoder.JSONDecodeError:
+                    msg = 'Host "{}" not supporting non-password authorization. Please, specify the password!'.format(
+                        login_url)
+                    raise AuthError(msg)
             if len(r.history) > 0:
                 for resp in r.history:
                     session = resp.cookies.get("session")
@@ -73,12 +74,18 @@ class Authorization:
         # отправляем запрос аутентификации на заданный URL
         r = requests.request(method="POST", url=url, json=params, timeout=60.0)
 
-        # проверки
-        session_id, uuid = self._authorization_checks(params, r.json(), r.status_code)
-
-        end_time = time.time()
-        func_timing = '{:s} func exec time: {:.2f} sec'.format("login", (end_time - start_time))
-        return session_id, uuid, func_timing
+        # проверки и вывод результата
+        status_code = r.status_code
+        if not r.ok:
+            msg = 'Invalid server response (URL: {}, Code: {}, Reason: "{}", Text: {})'.format(
+                url, status_code, r.reason, '<empty>' if not r.text else r.text)
+            raise AuthError(msg)
+        else:
+            try:
+                json_response = r.json()
+            except json.decoder.JSONDecodeError:
+                raise AuthError('Server response cannot be converted to the JSON format')
+            return self._authorization_checks(params, json_response, status_code)
 
     def _get_command(self, data: Dict) -> Dict:
         """
@@ -93,7 +100,7 @@ class Authorization:
         :param request: (Dict) запрос
         :param response: (Dict) ответ
         :param response_code: (int) код ответа
-        :return: (Tuple) идентификатор сессии, uuid
+        :return: (Tuple) идентификатор сессии, uuid, полная версия Полиматики
         """
         # получаем команды и коды запроса/ответа
         request_command = self._get_command(request)
@@ -110,6 +117,7 @@ class Authorization:
         # извлекаем идентификаторы
         session_id = resp_command.get("session_id")
         uuid = resp_command.get("manager_uuid")
+        polymatica_version = resp_command.get('version')
 
         # проверки идентификаторов
         assert session_id != "", "session_id == empty string"
@@ -117,4 +125,4 @@ class Authorization:
         assert uuid != "", "manager_id == empty string"
         assert uuid is not None, "manager_id is None!"
 
-        return session_id, uuid
+        return session_id, uuid, polymatica_version
