@@ -1,9 +1,11 @@
-# /usr/bin/python3
+#!/usr/bin/python3
 # -*- coding: utf-8 -*-
 """ Модуль обработки ошибок из ответа """
+
 import re
 import requests
 import datetime
+import os
 from typing import Dict, Tuple
 
 
@@ -54,29 +56,35 @@ def request_asserts(response: Dict, r: requests.models.Response) -> bool:
 
 def checks(self, func_name, *args):
     """
-    Проверки
-    :param self: экземпляр класса BusinessLogic
-    :param func_name: название функции
-    :param args: прочие параметры
+    Реализация проверок различных функций.
+    :param self: экземпляр класса BusinessLogic.
+    :param func_name: название функции.
+    :param args: прочие параметры, необходимые для проверки.
     :return:
     """
     if func_name == "rename_group":
         group_uuid = args[0]
         group_name = args[1]
+        new_name = args[2]
         if not group_uuid:
-            raise ValueError("No such group: %s " % group_name)
+            raise ValueError('No such group: {}'.format(group_name))
+        if not new_name:
+            raise ValueError('New group name cannot be empty!')
         return True
     if func_name == "move_dimension":
         if self.multisphere_module_id == "":
             raise ValueError("First create cube and get data from it!")
-        position = args[0]
-        if position == "left":
-            return 1
-        elif position == "up":
-            return 2
-        elif position == "out":
-            return 0
-        raise ValueError('Position "{}" does not exist! Position can only be "up", "left" or "out"!'.format(position))
+        # определяем числовое значение позиции
+        position, level = args[0], args[1]
+        num_position = {'left': 1, 'up': 2, 'out': 0}.get(position, -1)
+        # если осталось значение по-умолчанию - значит передана несуществующая позиция
+        if num_position == -1:
+            raise ValueError('Position "{}" does not exist! It can only be "up", "left" or "out"!'.format(position))
+        # при выносе размерности влево/вверх уровень должен быть явно задан
+        if num_position in [1, 2] and level is None:
+            raise ValueError('При выносе размерности влево/вверх должен быть явно задан параметр "level"!')
+        # вернём числовую позицию
+        return num_position
     elif func_name == "polymatica_health_check_multisphere_updates":
         cubes_list = args[0]
         cube_name = args[1]
@@ -84,23 +92,15 @@ def checks(self, func_name, *args):
             if cube["name"] == cube_name:
                 return True
         raise ValueError("No such cube in cubes list: %s" % cube_name)
-    elif func_name == "get_measure_name":
-        if self.multisphere_module_id == "":
-            raise ValueError("First create cube and get data from it!")
-        return True
     elif func_name == "rename_dimension":
         dim_name = args[0]
         if not isinstance(dim_name, str):
-            raise ValueError('Dimension copy name [%s] is not valid. It should have "string" type' % dim_name)
+            raise ValueError('Dimension name "{}" is not valid. It should have "string" type'.format(dim_name))
         return True
     elif func_name == "select_all_dims":
         left_dims = args[0]
         if not left_dims:
-            raise ValueError("Left dimension required")
-        return True
-    elif func_name == "get_dim_name":
-        if self.multisphere_module_id == "":
-            raise ValueError("First create cube and get data from it!")
+            raise ValueError("Left dimensions required!")
         return True
     elif func_name == "set_measure_visibility":
         is_visible = args[0]
@@ -109,20 +109,34 @@ def checks(self, func_name, *args):
         return True
     elif func_name == "sort_measure":
         is_visible = args[0]
-        if is_visible not in ("ascending", "descending"):
-            raise ValueError("sort_type can only equals 'ascending' OR 'descending'!")
+        if is_visible not in ("ascending", "descending", "off"):
+            raise ValueError('Param "sort_type" can only equals "ascending" or "descending" or "off"!')
         return True
     elif func_name == "unfold_all_dims":
         position = args[0]
+        level = args[1]
+        # проверка позиции
         if position == "left":
-            return 1
+            int_position = 1
         elif position == "up":
-            return 2
-        raise ValueError("Position %s does not exist! position can only be 'up' or 'left'!" % position)
+            int_position = 2
+        else:
+            raise ValueError('Position "{}" does not exist! Position can only be "up" or "left"!'.format(position))
+        # проверка значения уровня
+        if level < 0:
+            raise ValueError('Param "level" can be only positive!')
+        return int_position
     elif func_name == "set_width_columns":
-        measures, measures_list = args[0], args[1]
+        measures, measures_list, left_dims, left_dims_data = args[0], args[1], args[2], args[3]
+        error_msg = str()
         if len(measures) != len(measures_list):
-            raise ValueError("Количество колонок (параметр measures) должно совпадать с количеством фактов!")
+            error_msg = 'Длина списка в параметре "measures" должна совпадать с ' \
+                'количеством нескрытых фактов мультисферы!'
+        if len(left_dims) != len(left_dims_data):
+            error_msg = 'Длина списка в параметре "left_dims" должна совпадать с ' \
+                'количеством левых размерностей мультисферы!'
+        if error_msg:
+            raise ValueError(error_msg)
         return True
     elif func_name == "put_dim_filter":
         filter_name, start_date, end_date, months, week_days = args[0], args[1], args[2], args[3], args[4]
@@ -211,9 +225,11 @@ def checks(self, func_name, *args):
                 raise ValueError("Unknown date format!")
         return dates_list
     elif func_name == "export":
-        file_format = args[0]
+        file_format, file_path = args[0], args[1]
         if file_format not in ["csv", "xls", "json"]:
-            raise ValueError("Wrong file format: %s" % file_format)
+            raise ValueError('Wrong file format: "{}". Only .csv, .xls, .json formats allowed!'.format(file_format))
+        if not file_path:
+            raise ValueError('Empty file path!')
         return True
     elif func_name == "run_scenario":
         scenario_id, scenario_name = args[0], args[1]
@@ -299,5 +315,25 @@ def checks(self, func_name, *args):
             raise ValueError(error_msg)
         if not is_int or int(units) <= 0:
             raise ValueError(error_msg)
+    elif func_name == 'set_measure_level':
+        level, left_dims_count = args[0], args[1]
+        error_msg = str()
+        if left_dims_count < 3:
+            error_msg = '3 or more dimensions must be take out!'
+        elif level <= 0:
+            error_msg = 'Param "level" must be positive!'
+        elif level > left_dims_count - 1:
+            error_msg = 'Invalid "level" param! Max allowable value: {}'.format(left_dims_count - 1)
+        if error_msg:
+            raise ValueError(error_msg)
+    elif func_name == 'set_measure_select':
+        measure_id, measure_name = args[0], args[1]
+        if not measure_id and not measure_name:
+            raise ValueError('Need to specify either measure identifier or measure name!')
+    elif func_name == 'set_all_measure_visibility':
+        is_visible = args[0]
+        if not isinstance(is_visible, bool):
+            raise ValueError('Param "is_visible" can only be boolean: True / False')
+        return True
     else:
         raise ValueError("No function to check: %s" % func_name)
