@@ -1,14 +1,14 @@
-""" Модуль выполнения команд """
-# /usr/bin/python3
+#!/usr/bin/python3
 # -*- coding: utf-8 -*-
+""" Модуль исполнения команд к серверу Полиматики """
+
 import requests
 import json
 import logging
 from typing import Dict, Union
+from .error_handler import request_asserts
 
-from . import error_handler
-
-# Недопустимые управляюещие символы в json (взято из RFC 1345, начина с с символа 0000)
+# Недопустимые управляюещие символы в json (взято из RFC 1345, начиная с символа 0000)
 INVALID_CONTROL_CHARACTERS = ["\u0000", "\u0001", "\u0002", "\u0003", "\u0004", "\u0005", "\u0006", "\u0007", "\u0008",
                               "\u0009", "\u0010", "\u0011", "\u0012", "\u0013", "\u0014", "\u0015", "\u0016", "\u0017",
                               "\u0018", "\u0019",
@@ -148,7 +148,7 @@ INVALID_CONTROL_CHARACTERS = ["\u0000", "\u0001", "\u0002", "\u0003", "\u0004",
 
 
 class Executor:
-    """ Класс, выполняющий запросы """
+    """ Класс, выполняющий запросы к серверу Полиматики """
 
     def __init__(self, session_id: str, uuid: str, url: str, timeout: Union[str, float, int]):
         """
@@ -157,14 +157,21 @@ class Executor:
         :param uuid: (manager) uuid
         :param timeout: таймауты запросов
         """
-        self.query = {
-            "uuid": uuid,
-            "command": "your command"
-        }
         self.session_id = session_id
         self.base_url = url
         self.timeout = timeout
 
+    @staticmethod
+    def get_server_codes(base_url) -> Dict:
+        """
+        Получение GET-запросом списка команд и состояний, которые используются в Полиматике.
+        :param base_url: (str) URL-адрес стенда Полиматики.
+        :return: словарь списков и состояний, расположенный по пути base_url/server-codes.json.
+        """
+        r = requests.get('{}server-codes.json'.format(base_url))
+        assert r.status_code == 200, "Response code == {}!".format(r.status_code)
+        return r.json()
+
     def execute_request(self, params: Union[Dict, str], method: str = "POST") -> [
             requests.models.Response, Dict, str]:
         """
@@ -178,27 +185,20 @@ class Executor:
         # для избежания проблемы с чтением true/false (json-формат)
         json_params = json.dumps(params)
 
+        cookies = {'session': self.session_id}
+
         if method == "GET":
             # выкачать файл GET-запросом
-
-            # имя cookies: session (для скачивания файла)
-            cookies = {'session': self.session_id}
-
-            # выкачать файл GET-запросом
-            r = requests.request(method="GET", url=params, cookies=cookies, timeout=self.timeout)
-            return r
+            return requests.request(method="GET", url=params, cookies=cookies, timeout=self.timeout)
         elif method == "PUT":
-            cookies = {'session': self.session_id}
+            headers = {
+                'content-type': 'application/octet-stream',
+                'X-Requested-With': 'XMLHttpRequest',
+                'Upload-Position': "0",
+                'Last-Part': "1"
+            }
             with open(params, 'rb') as file:
-                r = requests.put(self.base_url + "/upload.php?",
-                                 data=file,
-                                 cookies=cookies,
-                                 headers={
-                                     'content-type': 'application/octet-stream',
-                                     'X-Requested-With': 'XMLHttpRequest',
-                                     'Upload-Position': "0",
-                                     'Last-Part': "1"
-                                        })
+                r = requests.put("{}upload.php?".format(self.base_url), data=file, cookies=cookies, headers=headers)
             return r
         else:
             # Request headers
@@ -291,15 +291,15 @@ class Executor:
                 response = r.json()
 
             except json.decoder.JSONDecodeError as e:
-                logging.exception("EXCEPTION!!! json.decoder.JSONDecodeError:")
+                logging.error("EXCEPTION!!! json.decoder.JSONDecodeError:")
                 logging.exception(e)
                 str_response = r.text
                 for i in INVALID_CONTROL_CHARACTERS:
                     str_response = str_response.replace(i, '')
                 json_acceptable_string = str_response.replace("'", "\"")
                 response = json.loads(json_acceptable_string)
-                error_handler.request_asserts(response, r)
+                request_asserts(response, r)
                 return response
 
-            error_handler.request_asserts(response, r)
+            request_asserts(response, r)
             return response
