@@ -1,21 +1,22 @@
-# /usr/bin/python3
+#!/usr/bin/python3
 # -*- coding: utf-8 -*-
-""" Часто используемые служебные функции """
+""" Содержит вспомогательные (служебные) методы, использующиеся в основном модуле работы с Полиматикой """
+
 from typing import List, Dict, Tuple
 from itertools import count
-import logging
 import time
-from .exceptions import ParseError, CubeNotFoundError, ScenarioError
+from .exceptions import ParseError, ScenarioError, PolymaticaException
+from .common import raise_exception
 
 class Helper:
     def __init__(self, sc):
         """
-        Инициализация класса GetDataChunk
+        Инициализация вспомогательного класса.
         :param sc: экземпляр класса BusinessLogic
         """
-        # экзмепляр класса BusinessLogic
         self.sc = sc
-        self.jupiter = sc.jupiter
+        # хранит функцию-генератор исключений
+        self._raise_exception = raise_exception(self.sc)
 
     def get_cube_id(self, cubes_list: List, cube_name: str) -> [str, bool]:
         """
@@ -27,7 +28,8 @@ class Helper:
         for cube in cubes_list:
             if cube["name"] == cube_name:
                 return cube["uuid"]
-        raise ValueError("No such cube \"{}\" in cubes list!".format(cube_name))
+        return self._raise_exception(
+            ValueError, 'No such cube "{}" in cubes list!'.format(cube_name), with_traceback=False)
 
     def get_measure_or_dim_id(self, multisphere_data: Dict, measure_dim: str, name) -> [str, bool]:
         """
@@ -40,47 +42,60 @@ class Helper:
         for item in multisphere_data[measure_dim]:
             if item["name"].rstrip() == name.rstrip():
                 return item["id"]
-        error_msg = "No such {}: {}".format(measure_dim[:-1], name)
-        logging.exception(error_msg)
-        logging.info("APPLICATION STOPPED")
-        if self.jupiter:
-            return "ERROR!!! {}".format(error_msg)
-        raise ValueError("ERROR!!! {}".format(error_msg))
+        error_msg = 'No such {}: "{}"'.format(measure_dim[:-1], name)
+        return self._raise_exception(ValueError, error_msg, with_traceback=False)
 
-    def get_dim_id(self, multisphere_data: Dict, name: str, cube_name: str) -> [str, bool]:
+    def get_dim_id(self, multisphere_data: Dict, name: str, cube_name: str) -> str:
         """
-        Получить id размерности
-        :param multisphere_data: рабочая область мультисферы
-        :param name: название размерности
-        :param cube_name: название текущей мультисферы
-        :return: id размерности
+        Получить идентификатор размерности по её названию.
+        :param multisphere_data: рабочая область мультисферы.
+        :param name: название размерности.
+        :param cube_name: название текущей мультисферы.
+        :return: Идентификатор размерности.
         """
         for item in multisphere_data["dimensions"]:
-            if item["name"].rstrip() == name.rstrip():
-                return item["id"]
-        logging.error("Dimension name [%s] is not valid for Multisphere [%s]", name, cube_name)
-        logging.info("APPLICATION STOPPED")
-        if self.jupiter:
-            return "ERROR!!! Dimension name [%s] is not valid for Multisphere [%s]" % (name, cube_name)
-        raise ValueError("ERROR!!! Dimension name [%s] is not valid for Multisphere [%s]" % (name, cube_name))
+            if item.get("name", '').rstrip() == name.rstrip():
+                return item.get("id")
+        error_msg = 'Dimension name "{}" is not valid for Multisphere "{}"!'.format(name, cube_name)
+        return self._raise_exception(ValueError, error_msg, with_traceback=False)
 
     def get_measure_id(self, multisphere_data: Dict, name: str, cube_name: str) -> [str, bool]:
         """
-        Получить id факта
-        :param multisphere_data: рабочая область мультисферы
-        :param name: имя факта
-        :param cube_name: название текущей мультисферы
-        :return: id факта
+        Получить идентификатор факта по его названию.
+        :param multisphere_data: информация по рабочей области мультисферы.
+        :param name: название факта.
+        :param cube_name: название текущей мультисферы.
+        :return: идентификатор факта.
         """
+        # поиск идентификатора факта по его названию
         name = name.strip()
-        for item in multisphere_data["facts"]:
-            if item["name"].strip() == name:
-                return item["id"]
-        logging.error("Measure name [%s] is not valid for Multisphere [%s]", name, cube_name)
-        logging.info("APPLICATION STOPPED")
-        if self.jupiter:
-            return "ERROR!!! Measure name [%s] is not valid for Multisphere [%s]" % (name, cube_name)
-        raise ValueError("ERROR!!! Measure name [%s] is not valid for Multisphere [%s]" % (name, cube_name))
+        for item in multisphere_data.get("facts"):
+            if item.get("name").strip() == name:
+                return item.get("id")
+        # если не найдено ничего - бросаем ошибку
+        error_msg = 'Measure name "{}" is not valid for Multisphere "{}"'.format(name, cube_name)
+        return self._raise_exception(PolymaticaException, error_msg, with_traceback=False)
+
+    def get_measure_or_dim_name_by_id(self, id_: str, type_: str) -> str:
+        """
+        Получить название факта/размерности по идентификатору.
+        :param id_: (str) идентификатор факта/размерности.
+        :param type_: (str) принимает одно из значений: "facts" или "dimensions".
+        :return: (str) название факта/размерности.
+        """
+        # проверка
+        if self.sc.multisphere_module_id == str():
+            return self._raise_exception(ValueError, 'First create cube and get data from it!', with_traceback=False)
+
+        # получить словать с размерностями, фактами и данными
+        self.sc.get_multisphere_data()
+
+        # поиск нужного имени
+        data = self.sc.multisphere_data.get(type_)
+        for item in data:
+            if item.get("id") == id_:
+                return item.get("name")
+        return 'No {} with id "{}" in the multisphere!'.format('measure' if type_ == 'facts' else 'dimension', id_)
 
     def get_measure_type(self, name: str) -> [bool, int]:
         """
@@ -106,12 +121,7 @@ class Helper:
             "UNKNOWN": 14
         }
         if name not in measure_types:
-            error_msg = "ERROR!!! No such measure type: {}".format(name)
-            logging.exception(error_msg)
-            logging.info("APPLICATION STOPPED")
-            if self.jupiter:
-                return error_msg
-            raise ValueError(error_msg)
+            return self._raise_exception(ValueError, "No such measure type: {}".format(name), with_traceback=False)
         return measure_types[name]
 
     def join_splited_measures(self, formula_lst: List, join_iterations: int):
@@ -183,55 +193,72 @@ class Helper:
         # рекурсивно вызвать join_splited_measures()
         return self.join_splited_measures(formula_lst, join_iterations)
 
-    def get_scenario_data(self, scenario_data: Dict, scenario_name: str) -> str:
+    def get_scenario_id_by_name(self, script_descs: Dict, scenario_name: str) -> str:
         """
-        Получение id сценария по имени сценария
-        :param scenario_data: (Dict) данные по всем сценариям, можно получить с помощью команды ("script", "list")
-        :param scenario_name: (str) название сценария
-        :return: id сценария
+        Получение идентификатор сценария по его имени.
+        :param script_descs: (Dict) данные по всем сценариям.
+        :param scenario_name: (str) название сценария.
+        :return: идентификатор сценария
         """
-        script_descs = self.parse_result(result=scenario_data, key="script_descs")
-        for i in script_descs:
-            if i.get("name") == scenario_name:
-                return i.get("uuid")
-        error_msg = 'Сценарий с именем "{}" не найден!'.format(scenario_name)
-        logging.error(error_msg, exc_info=True)
-        logging.info("APPLICATION STOPPED")
-        if self.jupiter:
-            return error_msg
-        raise ScenarioError(error_msg)
+        for script in script_descs:
+            if script.get("name") == scenario_name:
+                return script.get("uuid")
+        return self._raise_exception(
+            ScenarioError, 'Сценарий с именем "{}" не найден!'.format(scenario_name), with_traceback=False)
 
-    def wait_scenario_layer_loaded(self, sc_layer: str, timeout: int) -> [Tuple, str]:
+    def get_scenario_name_by_id(self, script_descs: Dict, scenario_id: str) -> str:
         """
-        Функция ожидает, когда слой с запущенным сценарием загрузится
-        :param self: экземпляр класса BusinessLogic
-        :param sc_layer: (str) id слоя со сценарием
-        :param timeout: (int) таймаут
-        :return: (tuple) общее число шагов сценария, число законченных шагов
+        Получение идентификатор сценария по его имени.
+        :param script_descs: (Dict) данные по всем сценариям.
+        :param scenario_id: (str) идентификатор сценария.
+        :return: название сценария
         """
-        start = time.time()
-        finished_steps, total_steps = 0, 1
-        while finished_steps < total_steps:
+        for script in script_descs:
+            if script.get("uuid") == scenario_id:
+                return script.get("name")
+        return self._raise_exception(
+            ScenarioError, 'Сценарий с идентификатором "{}" не найден!'.format(scenario_id), with_traceback=False)
+
+    def wait_scenario_layer_loaded(self, sc_layer: str) -> Tuple:
+        """
+        Ожидание загрузки слоя с заданным сценарием.
+        :param sc_layer: (str) идентификатор слоя с запускаемым сценарием.
+        :return: (Tuple) количество обращений к серверу для получения текущего статуса, число законченных шагов.
+        """
+        need_check_progress, count_of_requests, finished_steps = True, 0, 0
+        while need_check_progress:
+            # периодичностью раз в полсекунды запрашиваем результат с сервера и проверяем статус загрузки слоя
+            # если не удаётся получить статус - скорее всего нет ответа от сервера - сгенерируем ошибку
+            # в таком случае считаем, что сервер не ответил и генерируем ошибку
             time.sleep(0.5)
+            count_of_requests += 1
+            try:
+                progress = self.sc.execute_manager_command(
+                    command_name="script", state="run_progress", layer_id=sc_layer)
+                status = self.parse_result(result=progress, key="status") or {}
+                status_code, status_message = status.get('code', -1), status.get('message', 'Unknown error!')
+            except Exception as ex:
+                # если упала ошибка - не удалось получить ответ от сервера: возможно, он недоступен
+                return self._raise_exception(ScenarioError, 'Failed to load script! Possible server is unavailable.')
 
-            # check for timeout
-            end = time.time()
-            if (end - start) > timeout:
-                try:
-                    raise TimeoutError("Timeout while waiting loading scenario layer!")
-                except Exception as e:
-                    logging.exception("EXCEPTION!!! %s", e)
-                    logging.info("APPLICATION STOPPED")
-                    self.sc.current_exception = "EXCEPTION!!! %s" % e
-                    if self.jupiter:
-                        return self.sc.current_exception
-                    raise
-
-            progress = self.sc.execute_manager_command(command_name="script", state="run_progress", layer_id=sc_layer)
-            total_steps = self.parse_result(result=progress, key="total_steps_count")
-            finished_steps = self.parse_result(result=progress, key="finished_steps_count")
-
-        return total_steps, finished_steps
+            # проверяем код статуса
+            if status_code == 206:
+                # сценарий в процессе воспроизведения
+                need_check_progress = True
+            elif status_code == 207:
+                # сценарий полностью выполнен
+                need_check_progress = False
+            elif status_code == 208:
+                # ошибка: сценарий остановлен пользователем (довольно редкий случай)
+                return self._raise_exception(
+                    ScenarioError, 'Script loading was stopped by user!', with_traceback=False)
+            elif status_code == -1:
+                # ошибка: не удалось получить код текущего статуса
+                return self._raise_exception(ScenarioError, 'Unable to get status code!', with_traceback=False)
+            else:
+                # прочие ошибки
+                return self._raise_exception(ScenarioError, status_message, with_traceback=False)
+        return count_of_requests, self.parse_result(result=progress, key="finished_steps_count")
 
     def parse_result(self, result: Dict, key: str, nested_key: str = None):
         """
@@ -248,27 +275,18 @@ class Helper:
         value = request_command.get(key)
         if value is None:
             error_msg = "{}['{}']".format(base_error_msg, key)
-            logging.error(error_msg)
-            logging.info("APPLICATION STOPPED")
-            if self.jupiter:
-                return error_msg
-            raise ParseError(error_msg)
+            return self._raise_exception(ParseError, error_msg, with_traceback=False)
         if nested_key is not None:
             nested_value = value.get(nested_key)
             if nested_value is None:
                 error_msg = "{}['{}']['{}']".format(base_error_msg, key, nested_key)
-                logging.error(error_msg)
-                logging.info("APPLICATION STOPPED")
-                if self.jupiter:
-                    return error_msg
-                raise ParseError(error_msg)
+                return self._raise_exception(ParseError, error_msg, with_traceback=False)
             return nested_value
         return value
 
     def get_rows_cols(self, num_row: int = None, num_col: int = None) -> Dict:
         """
         Загрузить строки и колонки мультисферы
-        :param self: экземпляр класса BusinessLogic
         :param num_row: (int) количество строк мультисферы
         :param num_col: (int) количество колонок мультисферы
         :return: (Dict) command_name="view", state="get_2"
@@ -299,7 +317,6 @@ class Helper:
     def get_filter_rows(self, dim_id: str) -> Dict:
         """
         Загрузить строки и колонки мультисферы
-        :param self: экземпляр класса BusinessLogic
         :param dim_id: (str) id размерности
         :return: (Dict) command_name="view", state="get_2"
         """
